#-----------------------------------------------------------------------------
# Writer  v1
# 3/25/04
# Copyright (c) 2004 David G. Drumright (ockham)
#-----------------------------------------------------------------------------
# Makes two loops.
# First loop goes through the frames watching the Nib.
# Whenever the Nib is close enough to the paper (Y == paper Y)
# we create a copy of the Nib at the current location, and
# leave a record of the frame and name of copy.
# Reminder: the items will actually exist through the
# entire animation.
# So the Second loop simply goes through the record and forces
# the dots to be invisible (scale==0) until the point where
# they need to show up.
#-----------------------------------------------------------------------------

from Tkinter import *
import tkFileDialog
import poser
import math
import string
import os
scene = poser.Scene()
BasePathUncut=poser.AppLocation()
BasePath=BasePathUncut[:(-len('poser.exe'))]
NibPath=os.path.normpath(BasePath+'/Runtime/Libraries/props/Tiler/Nib.PP2')

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        self.master = master
        master.title("Writing")

        self.StatusEntry = Entry(self.master,width=40)
        self.StatusEntry.grid(row=0,column=1)
        # Check for animation frames
        if scene.NumFrames() < 2:
            self.ShowStatus("No animation frames.  Cancel and set up frames.")

        # Set up the Paper and Pen.
        self.Paper=0
        self.Pen=0
        for OneActor in scene.Actors():
            if OneActor.IsProp():
                UpName = string.upper(OneActor.Name())
                if UpName[:5] == 'PAPER':
                    self.Paper = OneActor
                elif UpName[:3] == 'NIB':
                    self.Pen = OneActor

        if not self.Paper:
            self.ShowStatus("Paper not present or improper.  Cancel and restart.")
        if not self.Pen:
            self.ShowStatus("Pen not present or improper.  Cancel and restart.")

        self.PaperGeom = self.Paper.Geometry()
        self.PenGeom = self.Pen.Geometry()

        self.ShowStatus("Pen and Paper seem to be present. Hit Go.")

        self.ButtonFrame = Frame(self.master)
        self.ButtonFrame.grid(row=2,column=0,pady=1)

        # Go/nogo buttons.
        self.buttonGo = Button(self.ButtonFrame, text="Go", command=self.HandleGo)
        self.buttonGo.grid(row=0, column=0)

        self.buttonCancel = Button(self.ButtonFrame, text="Close", command=self.HandleCancel)
        self.buttonCancel.grid(row=0, column=1)

        self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)

    # - - - - - - - - - - - - - - - - - -

    def ShowStatus(self,S): # Just saving repetition
        self.StatusEntry.delete(0,END)
        self.StatusEntry.insert(0,S)
        self.StatusEntry.update_idletasks()

    # - - - - - - - - - - - - - - - - - -
    def GetColors(self,Mat):
        self.TexCol = Mat.TextureColor()
        self.SpeCol = Mat.SpecularColor()
        self.DifCol = Mat.DiffuseColor()
        self.AmbCol = Mat.AmbientColor()
        self.RefCol = Mat.ReflectionColor()
        self.RefVal = Mat.ReflectionStrength()
        self.BumVal = Mat.BumpStrength()
        self.TrnMax = Mat.TransparencyMax()
        self.TrnMin = Mat.TransparencyMin()
        self.TrnFal = Mat.Ns()
        self.TexMap = Mat.TextureMapFileName()
        self.RefMap = Mat.ReflectionMapFileName()
        self.TrnMap = Mat.TransparencyMapFileName()
        self.BumMap = Mat.BumpMapFileName()

    # - - - - - - - - - - - - - - - - - -
    def PutColors(self,Mat):
        (R,G,B)=self.TexCol
        Mat.SetTextureColor(R,G,B)
        (R,G,B)=self.SpeCol
        Mat.SetSpecularColor(R,G,B)
        (R,G,B)=self.DifCol
        Mat.SetDiffuseColor(R,G,B)
        (R,G,B)=self.AmbCol
        Mat.SetAmbientColor(R,G,B)
        (R,G,B)=self.RefCol
        Mat.SetReflectionColor(R,G,B)
        Mat.SetReflectionStrength(self.RefVal)
        Mat.SetBumpStrength(self.BumVal)
        Mat.SetTransparencyMax(self.TrnMax)
        Mat.SetTransparencyMin(self.TrnMin)
        Mat.SetNs(self.TrnFal)
        Mat.SetTextureMapFileName(self.TexMap)
        Mat.SetReflectionMapFileName(self.RefMap)
        Mat.SetTransparencyMapFileName(self.TrnMap)
        Mat.SetBumpMapFileName(self.BumMap)
    # - - - - - - - - - - - - - - - - - -

    def HandleGo(self):
        self.ShowStatus("Working.......")
        PenGeom = self.Pen.Geometry()
        # Get the materials of the Pen, assuming the first one is the good one.
        Mat=self.Pen.Materials()[0]
        self.GetColors(Mat)
        # Get the paper's top Y.  We assume the paper is horizontal for now.
        PaperY = self.Paper.Parameter("yTran").Value()
        # Get the actual size of the Nib. (Can't use Scale because it will
        # be parented to something.)
        # Hmm.  Can we get this more easily by fetching the actual value
        # and dividing by WorldMatrix?
        MaxX=-10000.0
        MaxY=-10000.0
        MaxZ=-10000.0
        MinX= 10000.0
        MinY= 10000.0
        MinZ= 10000.0
        for Vindex in range(PenGeom.NumVertices()):
            WV = PenGeom.WorldVertex(Vindex)
            WVX = WV.X()
            WVY = WV.Y()
            WVZ = WV.Z()
            if   WVX > MaxX: MaxX = WVX
            elif WVX < MinX: MinX = WVX
            if   WVY > MaxY: MaxY = WVY
            elif WVY < MinY: MinY = WVY
            if   WVZ > MaxZ: MaxZ = WVZ
            elif WVZ < MinZ: MinZ = WVZ
        Xdim = MaxX - MinX
        Ydim = MaxY - MinY
        Zdim = MaxZ - MinZ
        # Divide this by empirical values of original size.
        OrigX=0.07
        OrigY=0.05
        OrigZ=0.07
        Xscale = Xdim / OrigX
        Yscale = Ydim / OrigY
        Zscale = Zdim / OrigZ
        # Try dividing these by the Paper's scale so they will be right after parenting.
        PaperXs = self.Paper.ParameterByCode(poser.kParmCodeXSCALE).Value()
        PaperYs = self.Paper.ParameterByCode(poser.kParmCodeYSCALE).Value()
        PaperZs = self.Paper.ParameterByCode(poser.kParmCodeZSCALE).Value()
        PaperAs = self.Paper.ParameterByCode(poser.kParmCodeASCALE).Value()
        Xscale = (Xdim / OrigX) / (PaperXs * PaperAs)
        Yscale = (Ydim / OrigY) / (PaperYs * PaperAs)
        Zscale = (Zdim / OrigZ) / (PaperZs * PaperAs)

        # First loop.
        # Go through the frames, watching the Nib.
        # Any time the Nib's Y is close to the PaperY, generate a copy of the
        # Nib, scaled to match.

        NibRecord = {} # Dict: key is name, value is frame.
        StartFrame = 0
        EndFrame = scene.NumFrames()
        for F in range(StartFrame,EndFrame):
            scene.SetFrame(F)
            scene.DrawAll()
            WV=PenGeom.WorldVertex(0) # 0 may not be lowest; check
            NibActualX = WV.X()
            NibActualY = WV.Y()
            NibActualZ = WV.Z()
            if abs(NibActualY - PaperY) < 0.005:
                # make a copy of Nib
                scene.LoadLibraryProp(NibPath)
                # The new prop will be currentactor at this moment.
                NewNib = scene.CurrentActor()
                # Scale it down or up, using the separate XYZ
                # I think we also need to set Ascale here, and record it.
                # Move it to the real Nib's location.  Reminder: We do a frame-switch here
                # because the dot needs to be at this location from the start, even though
                # we are acting at frame F.
                scene.SetFrame(0) # Put to start temporarily

                xt=NewNib.ParameterByCode(poser.kParmCodeXTRAN)
                xt.SetValue(NibActualX)

                yt=NewNib.ParameterByCode(poser.kParmCodeYTRAN)
                yt.SetValue(NibActualY-.0005)

                zt=NewNib.ParameterByCode(poser.kParmCodeZTRAN)
                zt.SetValue(NibActualZ)

                # Give it the nib's materials.
                Mat=NewNib.Materials()[0]
                self.PutColors(Mat)
                NewNib.Memorize()

                # Record the name and the frame when we made it
                NibRecord[NewNib.Name()]=F
                # Parent the dots to the paper.
                NewNib.SetParent(self.Paper,0,1)

        scene.DrawAll()

        # Second loop: go through the created nibs. For each,
        # scale to zero from start,
        # jump to full scale from the frame where created.
        self.ShowStatus("Yes, I'm still working... Just wait till I close.")
        for NewNibName in NibRecord.keys():
            ChangeFrame = NibRecord[NewNibName] # Get the point where it needs to jump
            NewNib = scene.Actor(NewNibName)
            xs=NewNib.ParameterByCode(poser.kParmCodeXSCALE)
            ys=NewNib.ParameterByCode(poser.kParmCodeYSCALE)
            zs=NewNib.ParameterByCode(poser.kParmCodeZSCALE)
            as=NewNib.ParameterByCode(poser.kParmCodeASCALE)
            # Up to ChangeFrame, set scale 0.
            for F in range(StartFrame,ChangeFrame):
                scene.SetFrame(F)
                xs.SetValue(Xscale)
                ys.SetValue(Yscale)
                zs.SetValue(Zscale)
                as.SetValue(0.0)
            # From ChangeFrame to end, set scale 1.
            for F in range(ChangeFrame,EndFrame):
                scene.SetFrame(F)
                xs.SetValue(Xscale)
                ys.SetValue(Yscale)
                zs.SetValue(Zscale)
                as.SetValue(1.0)

        scene.DrawAll()

        self.master.destroy() # Quit automatically

    # - - - - - - - - - - - - - - - - - -

    def HandleCancel(self):
        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
